import logging
import json
from tqdm import tqdm
import pandas as pd

logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(levelname)s - %(message)s'
)
logger = logging.getLogger(__name__)

def save_report(report: dict, path: str):
    with open(path, 'w') as f:
        json.dump(report, f, indent=2)
    logger.info(f"Report saved to {path}")


import argparse
from .pipeline import PreprocessingPipeline
from .text_preprocessor import TextPreprocessor

def main():
    parser = argparse.ArgumentParser(prog='dataprepkit', description='Data preprocessing toolkit')
    subparsers = parser.add_subparsers(dest='command')

    preprocess = subparsers.add_parser('preprocess', help='Run preprocessing')
    preprocess.add_argument('--input', required=True, help='Input CSV path')
    preprocess.add_argument('--output', required=True, help='Output CSV path')
    preprocess.add_argument('--text', help='Text column name')
    preprocess.add_argument('--steps', nargs='+', default=['clean', 'tokenize', 'lemmatize'],
                           help='Text steps: clean, tokenize, stopwords, lemmatize, spell')
    preprocess.add_argument('--chunksize', type=int, default=10000)
    preprocess.add_argument('--config', help='Pipeline YAML/JSON config')

    args = parser.parse_args()

    if args.command == 'preprocess':
        if args.config:
            pipeline = PreprocessingPipeline(args.config)
            pipeline.fit_transform(args.input, args.output, args.chunksize)
        elif args.text:
            tp = TextPreprocessor()
            tp.process_csv(args.input, args.text, args.output, steps=args.steps, chunksize=args.chunksize)
        else:
            print("Use --config or --text")
            return

if __name__ == '__main__':
    main()


import os
import re
import pandas as pd
from bs4 import BeautifulSoup
from typing import List, Dict, Any
from spellchecker import SpellChecker
import nltk
from nltk.corpus import stopwords, wordnet
from nltk.tokenize import word_tokenize
from nltk.stem import WordNetLemmatizer
from tqdm import tqdm
import logging

nltk.download('punkt', quiet=True)
nltk.download('stopwords', quiet=True)
nltk.download('wordnet', quiet=True)
nltk.download('averaged_perceptron_tagger', quiet=True)

logger = logging.getLogger("TextPreprocessor")
if not logger.handlers:
    handler = logging.StreamHandler()
    handler.setFormatter(logging.Formatter("%(message)s"))
    logger.addHandler(handler)
    logger.setLevel(logging.INFO)


class TextPreprocessor:
    def __init__(self, spell_correction: bool = False, verbose: bool = False):
        self.spell = SpellChecker() if spell_correction else None
        self.stop_words = set(stopwords.words('english'))
        self.lemmatizer = WordNetLemmatizer()
        self.verbose = verbose
        if verbose:
            logger.setLevel(logging.DEBUG)

        self.report = {
            "before": {"rows": 0, "words": 0},
            "after": {"rows": 0, "words": 0}
        }
    # Step Funciton
    def _clean_text(self, text: str) -> str:
        if not isinstance(text, str) or not text.strip():
            return ""
        text = BeautifulSoup(text, "html.parser").get_text()
        text = re.sub(r'http[s]?://\S+', '', text)           # URLs
        text = re.sub(r'[\U00010000-\U0010ffff]', '', text)  # Emojis
        text = re.sub(r'\s+', ' ', text).strip()
        return text

    def _remove_punctuation(self, text: str) -> str:
        return re.sub(r'[^\w\s]', ' ', text)

    def _remove_numbers(self, text: str) -> str:
        return re.sub(r'\d+', '', text)

    def _to_lowercase(self, text: str) -> str:
        return text.lower()

    def _tokenize(self, text: str) -> List[str]:
        return word_tokenize(text)

    def _remove_stopwords(self, tokens: List[str]) -> List[str]:
        return [t for t in tokens if t not in self.stop_words]

    def _lemmatize(self, tokens: List[str]) -> List[str]:
        def get_pos(tag):
            if tag.startswith('J'): return wordnet.ADJ
            if tag.startswith('V'): return wordnet.VERB
            if tag.startswith('N'): return wordnet.NOUN
            if tag.startswith('R'): return wordnet.ADV
            return wordnet.NOUN
        tagged = nltk.pos_tag(tokens)
        return [self.lemmatizer.lemmatize(w, get_pos(t)) for w, t in tagged]

    def _correct_spelling(self, tokens: List[str]) -> List[str]:
        if not self.spell:
            return tokens
        return [self.spell.correction(w) or w for w in tokens]

    def _filter_min_length(self, tokens: List[str], min_len: int = 3) -> List[str]:
        return [t for t in tokens if len(t) >= min_len]

    def process_text(self, text: str, steps: List[str]) -> str:

        if steps is None:
            steps = [
                'clean', 'punctuation', 'numbers', 'lowercase',
                'tokenize', 'stopwords', 'lemmatize', 'min_length'
            ]

        if not isinstance(text, str):
            return ""

        tokens = None  

        for step in steps:
            if step == 'clean':
                text = self._clean_text(text)
            elif step == 'punctuation':
                text = self._remove_punctuation(text)
            elif step == 'numbers':
                text = self._remove_numbers(text)
            elif step == 'lowercase':
                text = self._to_lowercase(text)
            elif step == 'tokenize':
                tokens = self._tokenize(text)
                text = ' '
            elif step == 'stopwords' and tokens is not None:
                tokens = self._remove_stopwords(tokens)
            elif step == 'lemmatize' and tokens is not None:
                tokens = self._lemmatize(tokens)
            elif step == 'spell' and tokens is not None and self.spell:
                tokens = self._correct_spelling(tokens)
            elif step == 'min_length' and tokens is not None:
                tokens = self._filter_min_length(tokens, min_len=3)

        return " ".join(tokens) if tokens is not None else text

    def process_csv(
        self,
        input_path: str,
        text_column: str,
        output_path: str,
        steps: List[str],
        chunksize: int = 10_000,
        encoding: str = "utf-8",
    ) -> Dict[str, Any]:
   
        if steps is None:
            steps = [
                'clean', 'punctuation', 'numbers', 'lowercase',
                'tokenize', 'stopwords', 'lemmatize', 'min_length'
            ]

        if not os.path.exists(input_path):
            raise FileNotFoundError(f"File not found: {input_path}")

        os.makedirs(os.path.dirname(output_path) or ".", exist_ok=True)

        # Count total rows
        with open(input_path, "r", encoding=encoding) as f:
            total_rows = sum(1 for _ in f) - 1
        logger.info(f"Processing {total_rows:,} rows → {output_path}")

        first_chunk = True
        processed = 0

        for chunk in tqdm(
            pd.read_csv(input_path, chunksize=chunksize, encoding=encoding),
            total=(total_rows // chunksize) + 1,
            desc="Cleaning",
            unit="chunk"
        ):
            # Stats before
            self.report["before"]["rows"] += len(chunk)
            self.report["before"]["words"] += chunk[text_column].astype(str).str.split().str.len().sum()

            if text_column not in chunk.columns:
                logger.error(f"Column '{text_column}' not found!")
                break

            chunk[text_column] = chunk[text_column].apply(
                lambda x: self.process_text(str(x), steps)
            )

            # Stats after
            self.report["after"]["rows"] += len(chunk)
            self.report["after"]["words"] += chunk[text_column].str.split().str.len().sum()

            # Write
            chunk.to_csv(
                output_path,
                mode="a",
                header=first_chunk,
                index=False,
                encoding=encoding
            )
            first_chunk = False
            processed += len(chunk)

        logger.info(f"Done! {processed:,} rows cleaned.")
        logger.info(
            f"Words: {self.report['before']['words']:,} → {self.report['after']['words']:,} "
            f"(-{self.report['before']['words'] - self.report['after']['words']:,})"
        )
        return self.report


import yaml
import json
import pandas as pd
from typing import Dict, Any
from .text_preprocessor import TextPreprocessor
from .numerical_preprocessor import NumericalPreprocessor
from .categorical_preprocessor import CategoricalPreprocessor
from .utils import logger, save_report, tqdm

class PreprocessingPipeline:
    def __init__(self, config_path: str):
        self.config = self._load_config(config_path)
        self.text_proc = TextPreprocessor(spell_correction=self.config.get('text', {}).get('spell', False))
        self.num_proc = NumericalPreprocessor()
        self.cat_proc = CategoricalPreprocessor(
            encoder_type=self.config.get('categorical', {}).get('type', 'label')
        )
        self.report = {"steps": [], "stats": {}}
        self.fitted = False

    def _load_config(self, path: str) -> Dict:
        with open(path, 'r') as f:
            if path.endswith('.yaml') or path.endswith('.yml'):
                return yaml.safe_load(f)
            else:
                return json.load(f)

    def fit(self, input_path: str, chunksize: int = 10000):
        first_chunk = pd.read_csv(input_path, nrows=1000)
        config = self.config

        if 'text' in config:
            col = config['text']['column']
            steps = config['text'].get('steps', ['clean', 'tokenize', 'lemmatize'])
            # Fit on sample
            self.text_proc.process_text(first_chunk[col].iloc[0], steps)

        if 'numerical' in config:
            cols = config['numerical']['columns']
            self.num_proc.fit(first_chunk, cols)

        if 'categorical' in config:
            cols = config['categorical']['columns']
            self.cat_proc.fit(first_chunk, cols)

        self.fitted = True
        logger.info("Pipeline fitted")

    def transform(self, input_path: str, output_path: str, chunksize: int = 10000):
        if not self.fitted:
            self.fit(input_path, chunksize)

        mode = 'w'
        header = True
        total_rows = sum(1 for _ in open(input_path)) - 1
        config = self.config

        with pd.read_csv(input_path, chunksize=chunksize) as reader:
            for chunk in tqdm(reader, total=(total_rows // chunksize) + 1, desc="Transforming"):
                if 'text' in config:
                    col = config['text']['column']
                    steps = config['text'].get('steps', [])
                    chunk[col] = chunk[col].astype(str).apply(
                        lambda x: self.text_proc.process_text(x, steps)
                    )

                if 'numerical' in config:
                    cols = config['numerical']['columns']
                    chunk = self.num_proc.transform(
                        chunk, cols,
                        impute=config['numerical'].get('impute'),
                        scale=config['numerical'].get('scale'),
                        remove_outliers=config['numerical'].get('outliers')
                    )

                if 'categorical' in config:
                    cols = config['categorical']['columns']
                    chunk = self.cat_proc.transform(chunk, cols)

                chunk.to_csv(output_path, mode=mode, header=header, index=False)
                mode = 'a'
                header = False

        save_report(self.report, "preprocessing_report.json")
        logger.info(f"Pipeline transform completed: {output_path}")

    def fit_transform(self, input_path: str, output_path: str, chunksize: int = 10000):
        self.fit(input_path, chunksize)
        self.transform(input_path, output_path, chunksize)


# src/TextPreprocessingToolkit/numerical_preprocessor.py
from __future__ import annotations

import pandas as pd
import numpy as np
from typing import Dict, Any, List, Literal, Optional
from sklearn.preprocessing import StandardScaler, MinMaxScaler


try:
    from .utils import logger
except ImportError:                     
    import logging
    logging.basicConfig(
        level=logging.INFO,
        format="%(asctime)s - %(levelname)s - %(message)s",
    )
    logger = logging.getLogger(__name__)


class NumericalPreprocessor:
    """
    Scikit-learn-style transformer for numeric columns.

    * Imputation (median / mean) – optional
    * Outlier removal (IQR / Z-score) – optional
    * Scaling (standard / minmax) – optional
    """

    def __init__(self) -> None:
        # will be created only when scaling is requested
        self.scaler: Optional[StandardScaler | MinMaxScaler] = None

        # medians / means learned during fit (used when the user asks for imputation without providing a value)
        self.impute_values: Dict[str, float] = {}

        self._fitted_columns: List[str] = []
        self.report: Dict[str, Any] = {"numerical": {}}

    def _detect_outliers_zscore(self, series: pd.Series, threshold: float = 3.0) -> pd.Series:
        """Return boolean mask where True = outlier (Z-score)."""
        if series.std() == 0:
            return pd.Series(False, index=series.index)
        z = np.abs((series - series.mean()) / series.std())
        return pd.Series(z > threshold, index=series.index, dtype=bool)

    def _detect_outliers_iqr(self, series: pd.Series) -> pd.Series:
        """Return boolean mask where True = outlier (IQR)."""
        Q1 = series.quantile(0.25)
        Q3 = series.quantile(0.75)
        IQR = Q3 - Q1
        lower = Q1 - 1.5 * IQR
        upper = Q3 + 1.5 * IQR
        return pd.Series((series < lower) | (series > upper), index=series.index, dtype=bool)

    def fit(
        self,
        df: pd.DataFrame,
        columns: List[str],
        *,
        impute: Literal["median", "mean"] | None = None,
    ) -> "NumericalPreprocessor":
 
        num_df = df[columns].select_dtypes(include=[np.number]).copy()
        self._fitted_columns = num_df.columns.tolist()

        if impute:
            for col in num_df.columns:
                if impute == "median":
                    self.impute_values[col] = float(num_df[col].median())
                else:  
                    self.impute_values[col] = float(num_df[col].mean())

        logger.info(
            f"NumericalPreprocessor fitted on {len(self._fitted_columns)} columns "
            f"(impute={impute})"
        )
        return self

    def transform(
        self,
        df: pd.DataFrame,
        columns: List[str] | None = None,
        *,
        impute: Literal["median", "mean"] | None = None,
        scale: Literal["standard", "minmax"] | None = None,
        remove_outliers: Literal["iqr", "zscore"] | None = None,
    ) -> pd.DataFrame:
    
        df = df.copy()
        cols = columns or self._fitted_columns
        import pandas.api.types as pd_types
        num_cols = [c for c in cols if c in df.columns and pd_types.is_numeric_dtype(df[c])]

        for col in num_cols:
            series = df[col].copy()

            # Imputation
            impute_val: Optional[float] = None
            if impute and series.isna().any():
                if impute == "median":
                    impute_val = series.median()
                else:  
                    impute_val = series.mean()

                # fallback if the column is still all-NaN after the above
                if pd.isna(impute_val):
                    impute_val = self.impute_values.get(col, 0.0)

                series.fillna(impute_val, inplace=True)
                self.report["numerical"].setdefault(col, {})["imputed_with"] = float(impute_val)

            if remove_outliers and series.notna().any():
                mask = (
                    self._detect_outliers_iqr(series)
                    if remove_outliers == "iqr"
                    else self._detect_outliers_zscore(series)
                )
                n_out = int(mask.sum())
                if n_out:
                    series[mask] = np.nan
                    # re-impute the holes we just created
                    if impute:
                        fill = impute_val if impute_val is not None else 0.0
                        if impute == "median":
                            fill = series.median() if pd.isna(fill) else fill
                        else:
                            fill = series.mean() if pd.isna(fill) else fill
                        if pd.isna(fill):
                            fill = 0.0
                        series.fillna(fill, inplace=True)
                    self.report["numerical"].setdefault(col, {})["outliers_removed"] = n_out

            df[col] = series

        if scale and num_cols:
            to_scale = df[num_cols]

            if scale == "standard":
                if self.scaler is None:                 
                    self.scaler = StandardScaler()
                    self.scaler.fit(to_scale)
                df[num_cols] = self.scaler.transform(to_scale)

            elif scale == "minmax":
                mm = MinMaxScaler()
                df[num_cols] = mm.fit_transform(to_scale)

        return df

    def fit_transform(
        self,
        df: pd.DataFrame,
        columns: List[str],
        *,
        impute: Literal["median", "mean"] | None = None,
        scale: Literal["standard", "minmax"] | None = None,
        remove_outliers: Literal["iqr", "zscore"] | None = None,
    ) -> pd.DataFrame:
        """
        Convenience method: learn statistics **and** apply the pipeline in one call.
        """
        return (
            self.fit(df, columns, impute=impute)
            .transform(df, columns, impute=impute, scale=scale, remove_outliers=remove_outliers)
        )

    def get_report(self) -> Dict[str, Any]:
        """Return a summary of what was done."""
        return self.report


from setuptools import setup, find_packages
from setuptools.command.install import install
import subprocess
import sys

class PostInstallCommand(install):
    def run(self):
        install.run(self)
        # Download NLTK resources after install
        try:
            subprocess.check_call([sys.executable, "-m", "nltk.downloader",
                                   "averaged_perceptron_tagger_eng", "wordnet",
                                   "stopwords", "punkt_tab"])
        except Exception as e:
            print(f"⚠️ Failed to download NLTK resources: {e}")

setup(
    name="TPTK",
    version="1.0.3",  
    author="Gaurav Jaiswal",
    author_email="jaiswalgaurav863@gmail.com",
    description="Automate text preprocessing tasks with ease.",
    long_description=open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/Gaurav-Jaiswal-1/Text-Preprocessing-Toolkit",
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    install_requires=[
        "nltk>=3.6.0",
        "pyspellchecker>=0.7.1",
        "pandas>=1.2.0",
    ],
    python_requires=">=3.8",
    cmdclass={
        "install": PostInstallCommand,
    },
)



import os
import joblib
from typing import Dict, Any

import pandas as pd
from sklearn.preprocessing import LabelEncoder, OneHotEncoder

try:
    from .utils import logger
except ImportError:                    
    import logging
    logging.basicConfig(level=logging.INFO)
    logger = logging.getLogger(__name__)

class CategoricalPreprocessor:
    def __init__(self, encoder_type: str = "label", save_dir: str = "encoders"):
        self.encoder_type = encoder_type
        self.save_dir = os.path.join(save_dir, encoder_type)   
        self.encoders: Dict[str, Any] = {}
        os.makedirs(self.save_dir, exist_ok=True)

    def fit(self, df: pd.DataFrame, columns: list):
        for col in columns:
            if col not in df.columns:
                continue

            if self.encoder_type == "label":
                enc = LabelEncoder()
                values_1d = df[col].astype(str).fillna("missing").values.ravel()
                enc.fit(values_1d)
                self.encoders[col] = enc
            else:   
                enc = OneHotEncoder(sparse_output=False, handle_unknown="ignore")
                enc.fit(df[[col]].fillna("missing"))
                self.encoders[col] = enc

            joblib.dump(enc, os.path.join(self.save_dir, f"{col}.pkl"))

        logger.info(f"Fitted {self.encoder_type} encoders for {len(self.encoders)} columns")
        return self

    def transform(self, df: pd.DataFrame, columns: list) -> pd.DataFrame:
        df = df.copy()
        for col in columns:
            if col not in self.encoders:
                logger.warning(f"No encoder for {col}")
                continue

            enc = self.encoders[col]

            if self.encoder_type == "label":
                vals = df[col].astype(str).fillna("missing").values.reshape(-1, 1)
                df[col] = enc.transform(vals).ravel()
            else:
                onehot = enc.transform(df[[col]].fillna("missing"))
                cats = enc.categories_[0]
                ohe_cols = [f"{col}_{c}" for c in cats]
                ohe_df = pd.DataFrame(onehot, columns=ohe_cols, index=df.index)
                df = pd.concat([df.drop(col, axis=1), ohe_df], axis=1)
        return df

    def load_encoders(self, columns: list):
        self.encoders = {}
        for col in columns:
            path = os.path.join(self.save_dir, f"{col}.pkl")
            if os.path.exists(path):
                self.encoders[col] = joblib.load(path)
            else:
                logger.warning(f"Encoder for {col} not found at {path}")
        logger.info(f"Loaded {len(self.encoders)} {self.encoder_type} encoders")


from .pipeline import PreprocessingPipeline
from .text_preprocessor import TextPreprocessor
from .numerical_preprocessor import NumericalPreprocessor
from .categorical_preprocessor import CategoricalPreprocessor

__version__ = "1.0.2"
__all__ = [
    "PreprocessingPipeline",
    "TextPreprocessor",
    "NumericalPreprocessor",
    "CategoricalPreprocessor"
]